diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/Makefile.kmk work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/Makefile.kmk
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/Makefile.kmk	2009-12-17 23:27:56.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/Makefile.kmk	2010-01-21 20:19:47.000000000 +0900
@@ -52,6 +52,12 @@
 VBoxHeadless_TEMPLATE  := $(if $(VBOX_WITH_HARDENING),VBOXMAINCLIENTDLL,VBOXMAINCLIENTEXE)
 VBoxHeadless_DEFS       = VBOX_FFMPEG
 VBoxHeadless_SOURCES    = VBoxHeadless.cpp
+ifdef VBOX_WITH_LIBVNC
+ VBoxHeadless_DEFS     += VBOX_WITH_LIBVNC
+ VBoxHeadless_SOURCES  += VNCFramebuffer.cpp
+ VBoxHeadless_SOURCES  += keymap.c
+ VBoxHeadless_LIBS     += vncserver z jpeg
+endif
 ifdef VBOX_WITH_VRDP
  VBoxHeadless_DEFS     += VBOX_WITH_VRDP
  VBoxHeadless_SOURCES  += Framebuffer.cpp
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VBoxHeadless.cpp work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VBoxHeadless.cpp
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VBoxHeadless.cpp	2009-12-17 23:27:56.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VBoxHeadless.cpp	2010-01-23 18:55:09.000000000 +0900
@@ -47,6 +47,17 @@
 #include <VBox/err.h>
 #include <VBox/VBoxVideo.h>
 
+#ifdef VBOX_WITH_LIBVNC
+#ifdef VBOX_FFMPEG
+#undef VBOX_FFMPEG
+#endif
+#include "VNCFramebuffer.h"
+#ifdef __FreeBSD__
+#include <sys/types.h>
+#include <unistd.h>
+#endif /* __FreeBSD__ */
+#endif /* VBOX_WITH_LIBVNC */
+
 #ifdef VBOX_FFMPEG
 #include <cstdlib>
 #include <cerrno>
@@ -76,6 +87,11 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 
+#ifdef VBOX_WITH_LIBVNC
+static BOOL gfAbsoluteMouseGuest = FALSE;
+static BOOL gfGuestNeedsHostCursor = FALSE;
+#endif /* VBOX_WITH_LIBVNC */
+
 /* global weak references (for event handlers) */
 static ComPtr <ISession, ComWeakRef> gSession;
 static ComPtr <IConsole, ComWeakRef> gConsole;
@@ -145,6 +161,10 @@
 
     STDMETHOD(OnMouseCapabilityChange) (BOOL supportsAbsolute, BOOL needsHostCursor)
     {
+#ifdef VBOX_WITH_LIBVNC
+        gfAbsoluteMouseGuest = supportsAbsolute;
+        gfGuestNeedsHostCursor = needsHostCursor;
+#endif /* VBOX_WITH_LIBVNC */
         /* Emit absolute mouse event to actually enable the host mouse cursor. */
         if (supportsAbsolute && gConsole)
         {
@@ -356,6 +376,13 @@
 {
     RTPrintf("Usage:\n"
              "   -s, -startvm, --startvm <name|uuid>   Start given VM (required argument)\n"
+#ifdef VBOX_WITH_LIBVNC
+             "   -v, -vnc, --vnc on|off                Enable (default) or disable the VNC Server\n"
+             "   -a, -vncaddress, --vncaddress <ip>    IP address the VNC server will bind to\n"
+             "   -p, -vncport, --vncport <port>        Port number the VNC server will bind to\n"
+             "   -k, -vnckeymap, --vnckeymap <keymap>  Keyboard mapfile (default: builtin US)\n"
+             "   -S, -vncsecret, --vncsecret <secret>  VNC Authentication secret\n"
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_WITH_VRDP
              "   -v, -vrdp, --vrdp on|off|config       Enable (default) or disable the VRDP\n"
              "                                         server or don't change the setting\n"
@@ -427,6 +454,13 @@
  */
 extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
 {
+#ifdef VBOX_WITH_LIBVNC
+    unsigned int vncPort = 5900;
+    const char *vncAddress = NULL;
+    const char *vncEnabled = NULL;
+    const char *vncKeymap = NULL;
+    const char *vncSecret = NULL;
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_WITH_VRDP
     const char *vrdpPort = NULL;
     const char *vrdpAddress = NULL;
@@ -482,6 +516,18 @@
     {
         { "-startvm", 's', RTGETOPT_REQ_STRING },
         { "--startvm", 's', RTGETOPT_REQ_STRING },
+#ifdef VBOX_WITH_LIBVNC
+        { "-vncport", 'p', RTGETOPT_REQ_UINT32 },
+        { "--vncport", 'p', RTGETOPT_REQ_UINT32 },
+        { "-vncaddress", 'a', RTGETOPT_REQ_STRING },
+        { "--vncaddress", 'a', RTGETOPT_REQ_STRING },
+        { "-vnc", 'v', RTGETOPT_REQ_STRING },
+        { "--vnc", 'v', RTGETOPT_REQ_STRING },
+        { "-vnckeymap", 'k', RTGETOPT_REQ_STRING },
+        { "--vnckeymap", 'k', RTGETOPT_REQ_STRING },
+        { "-vncsecret", 'S', RTGETOPT_REQ_STRING },
+        { "--vncsecret", 'S', RTGETOPT_REQ_STRING },
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_WITH_VRDP
         { "-vrdpport", 'p', RTGETOPT_REQ_STRING },
         { "--vrdpport", 'p', RTGETOPT_REQ_STRING },
@@ -533,6 +579,23 @@
                 if (!id)
                     name = ValueUnion.psz;
                 break;
+#ifdef VBOX_WITH_LIBVNC
+            case 'p':
+                vncPort = ValueUnion.u32;
+                break;
+            case 'a':
+                vncAddress = ValueUnion.psz;
+                break;
+            case 'v':
+                vncEnabled = ValueUnion.psz;
+                break;
+            case 'k':
+                vncKeymap = ValueUnion.psz;
+                break;
+            case 'S':
+                vncSecret = ValueUnion.psz;
+                break;
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_WITH_VRDP
             case 'p':
                 vrdpPort = ValueUnion.psz;
@@ -610,6 +673,15 @@
                 return -1;
         }
     }
+#ifdef VBOX_WITH_LIBVNC
+#ifdef __FreeBSD__
+    if (vncSecret != NULL) {
+	setproctitle("VM: %s Port: %d Auth: on", (name != NULL) ? name : "none", vncPort);
+    } else {
+	setproctitle("VM: %s Port: %d Auth: off", (name != NULL) ? name : "none", vncPort);
+    }
+#endif /* __FreeBSD__ */
+#endif /* VBOX_WITH_LIBVNC */
 
 #ifdef VBOX_FFMPEG
     if (ulFrameWidth < 512 || ulFrameWidth > 2048 || ulFrameWidth % 2)
@@ -856,6 +928,49 @@
                 break;
         }
 
+#ifdef VBOX_WITH_LIBVNC
+	BOOL fVNCEnabled = true;
+	VBoxVNCFB *pFramebuffer = NULL;
+        unsigned uScreenId = 0;
+
+	if (vncEnabled != NULL)
+	{
+            /* -vnc on|off */
+            if (!strcmp(vncEnabled, "off") || !strcmp(vncEnabled, "disable"))
+                fVNCEnabled = false;
+            else if (strcmp(vncEnabled, "on") && strcmp(vncEnabled, "enable"))
+            {
+                RTPrintf("-vnc requires an argument (on|off)\n");
+                break;
+            }
+        }
+
+        if (fVNCEnabled)
+        {
+            Log (("VBoxHeadless: Enabling VNC server...\n"));
+
+            pFramebuffer = new VBoxVNCFB ();
+            if (!pFramebuffer)
+            {
+                RTPrintf("Error: failed to allocate framebuffer\n");
+                break;
+            }
+            if (!pFramebuffer->loadKeymap(vncKeymap)) {
+                RTPrintf("Error: failed to load keymap\n");
+                break;
+	    }
+            pFramebuffer->setAuthInfo(vncSecret);
+            if (!pFramebuffer->init(name, vncPort, vncAddress))
+            {
+                RTPrintf("Error: failed to initialize framebuffer\n");
+                break;
+            }
+            pFramebuffer->setMouseAbsolute(&gfAbsoluteMouseGuest, &gfGuestNeedsHostCursor);
+            pFramebuffer->setConsole(console);
+            pFramebuffer->AddRef();
+            display->SetFramebuffer(uScreenId, pFramebuffer);
+        }
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_WITH_VRDP
         /* default is to enable the RDP server (backward compatibility) */
         BOOL fVRDPEnable = true;
@@ -950,6 +1065,14 @@
 
         Log (("VBoxHeadless: event loop has terminated...\n"));
 
+#ifdef VBOX_WITH_LIBVNC
+        if (pFramebuffer)
+        {
+            pFramebuffer->Release ();
+            Log(("Released framebuffer\n"));
+            pFramebuffer = NULL;
+        }
+#endif /* VBOX_WITH_LIBVNC */
 #ifdef VBOX_FFMPEG
         if (pFramebuffer)
         {
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.cpp work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.cpp
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.cpp	1970-01-01 09:00:00.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.cpp	2010-01-23 22:22:17.000000000 +0900
@@ -0,0 +1,490 @@
+/*
+ * VNCFramebuffer with LibVNCServer
+ *   designed for FreeNAS 0.7.1
+ * Created by Daisuke Aoyama <aoyama@peach.ne.jp>
+ */
+
+#include <VBox/com/com.h>
+#include <VBox/com/string.h>
+#include <VBox/com/Guid.h>
+#include <VBox/com/ErrorInfo.h>
+#include <VBox/com/EventQueue.h>
+#include <VBox/com/VirtualBox.h>
+
+#include <iprt/stream.h>
+#include <iprt/env.h>
+
+#include <ctype.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
+using namespace com;
+
+#define LOG_GROUP LOG_GROUP_GUI
+#include <VBox/err.h>
+#include <VBox/log.h>
+
+#include "VBoxHeadless.h"
+#include "VNCFramebuffer.h"
+#include "keymap.h"
+
+#if defined(VBOX_WITH_XPCOM)
+NS_IMPL_ISUPPORTS1_CI(VBoxVNCFB, IFramebuffer)
+NS_DECL_CLASSINFO(VBoxVNCFB)
+#endif
+
+static rfbBool dopwcheck(rfbClientPtr cl, const char* response, int len);
+static void doptr(int buttonMask, int x, int y, rfbClientPtr cl);
+static void dokbd(rfbBool down, rfbKeySym keySym, rfbClientPtr cl);
+
+VBoxVNCFB::VBoxVNCFB()
+{
+    int rc;
+    LogFlow(("VBoxFNCFB::VBoxVNCFB\n"));
+    rc = RTCritSectInit(&m_UpdateLock);
+    AssertMsg(rc == VINF_SUCCESS, ("Error from RTCritSectInit!\n"));
+    m_Screen = NULL;
+    m_secret = NULL;
+    m_AbsoluteMouseGuest = NULL;
+    m_GuestNeedsHostCursor = NULL;
+    m_Keyboard = NULL;
+    m_Mouse = NULL;
+    m_ptrx = m_ptry = -1;
+}
+
+VBoxVNCFB::~VBoxVNCFB()
+{
+    LogFlow(("VBoxVNCFB::~VBoxVNCFB\n"));
+    if (m_Screen != NULL) {
+	rfbShutdownServer(m_Screen, TRUE);
+	free((void*)m_Screen->desktopName);
+	free((void*)m_Screen->frameBuffer);
+	rfbScreenCleanup(m_Screen);
+	m_Screen = NULL;
+    }
+    RTCritSectDelete(&m_UpdateLock);
+}
+
+bool VBoxVNCFB::init(const char *name, unsigned int port, const char *address)
+{
+    int width = 640, height = 480, bps = 8, spp = 3, bpp =4;
+    int rmax = 255, gmax = 255, bmax = 255;
+    int rshift = 8, gshift = 8, bshift = 8;
+    int dname_len;
+    char *dname;
+
+    m_Screen = rfbGetScreen(NULL, NULL, width, height, bps, spp, bpp);
+    dname_len = 64 + (name != NULL ? strlen(name) : 0) + 1;
+    dname = (char*)malloc(dname_len);
+    if (name != NULL) {
+	snprintf(dname, dname_len, "VBox (%s)", name);
+    } else {
+	snprintf(dname, dname_len, "VBox");
+    }
+    m_Screen->desktopName = dname;
+    m_Screen->port = port;
+    if (address == NULL
+	|| strcmp(address, "*") == 0
+	|| strcmp(address, "0.0.0.0") == 0) {
+	m_Screen->listenInterface = htonl(INADDR_ANY);
+    } else {
+	m_Screen->listenInterface = inet_addr(address);
+    }
+    m_Screen->frameBuffer = (char*)malloc(width * height * bpp);
+    memset(m_Screen->frameBuffer, 0, (width * height * bpp));
+    m_Screen->ptrAddEvent = doptr;
+    m_Screen->kbdAddEvent = dokbd;
+    m_Screen->screenData = (void*)this;
+    m_Screen->authPasswdData = (void*)m_secret;
+    m_Screen->passwordCheck = dopwcheck;
+    rfbInitServer(m_Screen);
+    m_Screen->serverFormat.redMax = rmax;
+    m_Screen->serverFormat.greenMax = gmax;
+    m_Screen->serverFormat.blueMax = bmax;
+    m_Screen->serverFormat.redShift = rshift;
+    m_Screen->serverFormat.greenShift = gshift;
+    m_Screen->serverFormat.blueShift = bshift;
+    rfbRunEventLoop(m_Screen, -1, TRUE);
+
+    return true;
+}
+
+bool VBoxVNCFB::loadKeymap(const char *keymap)
+{
+    if (load_keymap_table(keymap) != 0) {
+	return false;
+    }
+    return true;
+}
+
+void VBoxVNCFB::setAuthInfo(const char *secret)
+{
+    m_secret = secret;
+}
+
+void VBoxVNCFB::setMouseAbsolute(BOOL *AbsoluteMouseGuest, BOOL *GuestNeedsHostCursor)
+{
+    m_AbsoluteMouseGuest = AbsoluteMouseGuest;
+    m_GuestNeedsHostCursor = GuestNeedsHostCursor;
+}
+
+void VBoxVNCFB::setConsole(ComPtr<IConsole> &Console)
+{
+    Console->COMGETTER(Keyboard)(m_Keyboard.asOutParam());
+    AssertMsg(!m_Keyboard, ("Error keyboard\n"));
+    Console->COMGETTER(Mouse)(m_Mouse.asOutParam());
+    AssertMsg(!m_Mouse, ("Error mouse\n"));
+}
+
+static rfbBool dopwcheck(rfbClientPtr cl, const char* response, int len)
+{
+    VBoxVNCFB *fb = (VBoxVNCFB*)cl->screen->screenData;
+    char *secret = (char*)fb->getAuthInfo();
+
+#if 0
+    printf("secret=%s\n", secret);
+#endif
+    if (secret == NULL || secret[0] == '\0')
+	return FALSE;
+    rfbEncryptBytes(cl->authChallenge, secret);
+    if (memcmp(cl->authChallenge, response, len) != 0) {
+	rfbErr("authentication failed from %s\n", cl->host);
+	return FALSE;
+    }
+    return TRUE;
+}
+
+static void doptr(int buttonMask, int x, int y, rfbClientPtr cl)
+{
+    VBoxVNCFB *fb = (VBoxVNCFB*)cl->screen->screenData;
+    ComPtr<IMouse> Mouse = fb->getMouse();
+    bool abs = fb->getMouseAbsolute();
+    int dx = 0, dy = 0, dz = 0, buttons = 0;
+
+#if 0
+    printf("ptr: x=%d, y=%d, mask=%d\n", x, y, buttonMask);
+#endif
+    if (buttonMask & 0x01)
+	buttons |= MouseButtonState_LeftButton;
+    if (buttonMask & 0x02)
+	buttons |= MouseButtonState_MiddleButton;
+    if (buttonMask & 0x04)
+	buttons |= MouseButtonState_RightButton;
+    if (buttonMask & 0x08)
+	dz = -1;
+    if (buttonMask & 0x10)
+	dz = 1;
+
+    if (abs) {
+	Mouse->PutMouseEventAbsolute(x, y, dz, 0, buttons);
+    } else {
+	int ptrx, ptry;
+	fb->getMousePtr(ptrx, ptry);
+	if (ptrx != -1 && ptry != -1) {
+	    dx = ((x - ptrx) * 160) / 100;
+	    dy = ((y - ptry) * 160) / 100;
+	    Mouse->PutMouseEvent(dx, dy, dz, 0, buttons);
+	}
+	fb->setMousePtr(x, y);
+    }
+}
+
+static void dokbd(rfbBool down, rfbKeySym keySym, rfbClientPtr cl)
+{
+    VBoxVNCFB *fb = (VBoxVNCFB*)cl->screen->screenData;
+    ComPtr<IKeyboard> Keyboard = fb->getKeyboard();
+    int scancode;
+
+#if 0
+    printf("down=%d, Sym=%x\n", down ? 1 : 0, keySym);
+#endif
+    if (keySym < 0x100 && isalpha(keySym)) {
+	keySym = tolower(keySym);
+    }
+    switch (keySym) {
+    case XK_Alt_R:
+	scancode = keysym2scancode(XK_Alt_L);
+	Keyboard->PutScancode(0xe0);
+	Keyboard->PutScancode(!down ? scancode | 0x80 : scancode);
+	break;
+    case XK_Control_R:
+	scancode = keysym2scancode(XK_Control_L);
+	Keyboard->PutScancode(0xe0);
+	Keyboard->PutScancode(!down ? scancode | 0x80 : scancode);
+	break;
+    case XK_Print:
+	scancode = 0x2a;
+	Keyboard->PutScancode(0xe0);
+	Keyboard->PutScancode(!down ? scancode | 0x80 : scancode);
+	scancode = 0x37;
+	Keyboard->PutScancode(0xe0);
+	Keyboard->PutScancode(!down ? scancode | 0x80 : scancode);
+	break;
+#if 1
+    case XK_KP_0:
+	keySym = '0'; goto normal_scancode;
+    case XK_KP_1:
+	keySym = '1'; goto normal_scancode;
+    case XK_KP_2:
+	keySym = '2'; goto normal_scancode;
+    case XK_KP_3:
+	keySym = '3'; goto normal_scancode;
+    case XK_KP_4:
+	keySym = '4'; goto normal_scancode;
+    case XK_KP_5:
+	keySym = '5'; goto normal_scancode;
+    case XK_KP_6:
+	keySym = '6'; goto normal_scancode;
+    case XK_KP_7:
+	keySym = '7'; goto normal_scancode;
+    case XK_KP_8:
+	keySym = '8'; goto normal_scancode;
+    case XK_KP_9:
+	keySym = '9'; goto normal_scancode;
+    case XK_KP_Decimal:
+	keySym = '.'; goto normal_scancode;
+#endif
+    default:
+	;
+    normal_scancode:
+	scancode = keysym2scancode(keySym);
+	if (scancode < 0) {
+	    //printf("unknown code %x\n", keySym);
+	} else {
+	    //printf("scan code %x => %x\n", keySym, scancode);
+	    scancode &= 0x7f;
+	    Keyboard->PutScancode(!down ? scancode | 0x80 : scancode);
+	}
+    }
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(Width)(ULONG *width)
+{
+    LogFlow(("VBoxVNCFB::GetWidth\n"));
+    if (!width)
+        return E_INVALIDARG;
+    *width = (ULONG)m_Screen->width;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(Height)(ULONG *height)
+{
+    LogFlow(("VBoxVNCFB::GetHeight\n"));
+    if (!height)
+        return E_INVALIDARG;
+    *height = (ULONG)m_Screen->height;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::Lock()
+{
+    LogFlow(("VBoxVNCFB::Lock\n"));
+    RTCritSectEnter(&m_UpdateLock);
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::Unlock()
+{
+    LogFlow(("VBoxVNCFB::Unlock\n"));
+    RTCritSectLeave(&m_UpdateLock);
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(Address)(BYTE **address)
+{
+    LogFlow(("VBoxVNCFB::GetAddress\n"));
+    if (!address)
+        return E_INVALIDARG;
+    *address = (BYTE*)m_Screen->frameBuffer;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(BitsPerPixel)(ULONG *bitsPerPixel)
+{
+    LogFlow(("VBoxVNCFB::GetBitsPerPixel\n"));
+    if (!bitsPerPixel)
+        return E_INVALIDARG;
+    *bitsPerPixel = (ULONG)m_Screen->bitsPerPixel;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(BytesPerLine)(ULONG *bytesPerLine)
+{
+    LogFlow(("VBoxVNCFB::GetBytesPerLine\n"));
+    if (!bytesPerLine)
+        return E_INVALIDARG;
+    *bytesPerLine = (ULONG)(m_Screen->width * (m_Screen->bitsPerPixel / 8));
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(PixelFormat) (ULONG *pixelFormat)
+{
+    if (!pixelFormat)
+        return E_POINTER;
+    *pixelFormat = FramebufferPixelFormat_FOURCC_RGB;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(UsesGuestVRAM) (BOOL *usesGuestVRAM)
+{
+    if (!usesGuestVRAM)
+        return E_POINTER;
+    *usesGuestVRAM = FALSE;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(HeightReduction) (ULONG *heightReduction)
+{
+    if (!heightReduction)
+        return E_POINTER;
+    *heightReduction = 0;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(Overlay) (IFramebufferOverlay **aOverlay)
+{
+    if (!aOverlay)
+        return E_POINTER;
+    *aOverlay = 0;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::COMGETTER(WinId) (uint64_t *winId)
+{
+    if (!winId)
+        return E_POINTER;
+    *winId = 0;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::NotifyUpdate(ULONG x, ULONG y, ULONG w, ULONG h)
+{
+    ULONG x2, y2;
+
+    if (w == 0)
+	w = 1;
+    if (h == 0)
+	h = 1;
+    x2 = x + w;
+    y2 = y + h;
+
+    if (x >= (ULONG)m_Screen->width)
+	x = (ULONG)m_Screen->width;
+    if (x2 >= (ULONG)m_Screen->width)
+	x2 = (ULONG)m_Screen->width;
+    if (y >= (ULONG)m_Screen->height)
+	y = (ULONG)m_Screen->height;
+    if (y2 >= (ULONG)m_Screen->height)
+	y2 = (ULONG)m_Screen->height;
+#if 0
+    printf("Update X=%d, Y=%d, W=%d, H=%d =>", x, y, w, h);
+    printf("X=%d, Y=%d, X2=%d, Y2=%d\n", x, y, x2, y2);
+#endif
+    rfbMarkRectAsModified(m_Screen, x, y, x2, y2);
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *vram,
+                                   ULONG bitsPerPixel, ULONG bytesPerLine,
+                                   ULONG w, ULONG h, BOOL *finished)
+{
+    if (!finished)
+	return E_FAIL;
+#if 0
+    AssertReturn(pixelFormat == FramebufferPixelFormat_FOURCC_RGB, E_INVALIDARG);
+    AssertReturn(bitsPerPixel == 32, E_INVALIDARG);
+#endif
+
+    printf("RequestResize: w=%d, h=%d, bpp=%d, bpl=%d, fmt=%x\n",
+	   w, h, bitsPerPixel, bytesPerLine, pixelFormat);
+
+    int width = w, height = h, bps = 8, spp = 3, bpp =4;
+    int rmax = 255, gmax = 255, bmax = 255;
+    int rshift = 8, gshift = 8, bshift = 8;
+    char *oldfb, *newfb;
+
+    switch (bitsPerPixel) {
+    case 32: /* X888 */
+	bps = 8; spp = 3; bpp = 4;
+	rmax = 255; gmax = 255; bmax = 255;
+	rshift = 16; gshift = 8; bshift = 0;
+	break;
+    case 24: /* 888 */
+	bps = 8; spp = 3; bpp = 4;
+	rmax = 255; gmax = 255; bmax = 255;
+	rshift = 16; gshift = 8; bshift = 0;
+	break;
+    case 16: /* 565 */
+	bps = 8; spp = 2; bpp = 2;
+	rmax = 32; gmax = 64; bmax = 32;
+	rshift = 11; gshift = 5; bshift = 0;
+	break;
+    case 8: /* 332 */
+	bps = 8; spp = 1; bpp = 1;
+	rmax = 7; gmax = 7; bmax = 3;
+	rshift = 5; gshift = 2; bshift = 0;
+	break;
+    case 0: /* XXX */
+    default:
+	bps = 8; spp = 3; bpp = 4;
+	rmax = 255; gmax = 255; bmax = 255;
+	rshift = 16; gshift = 8; bshift = 0;
+	break;
+    }
+#if 0
+    printf("bps=%d, spp=%d, bpp=%d, r=%d, g=%d, b=%d\n",
+	   bps, spp, bpp, rmax, gmax, bmax);
+#endif
+
+    oldfb = m_Screen->frameBuffer;
+    newfb = (char*)malloc(width * height * bpp);
+    memset(newfb, 0, (width * height * bpp));
+#if 0
+    rfbMarkRectAsModified(m_Screen, 0, 0, m_Screen->width, m_Screen->height);
+#endif
+    rfbNewFramebuffer(m_Screen, newfb, width, height, bps, spp, bpp);
+    m_Screen->serverFormat.redMax = rmax;
+    m_Screen->serverFormat.greenMax = gmax;
+    m_Screen->serverFormat.blueMax = bmax;
+    m_Screen->serverFormat.redShift = rshift;
+    m_Screen->serverFormat.greenShift = gshift;
+    m_Screen->serverFormat.blueShift = bshift;
+    free(oldfb);
+
+    *finished = true;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::VideoModeSupported(ULONG width, ULONG height, ULONG bpp, BOOL *supported)
+{
+    if (!supported)
+        return E_POINTER;
+    *supported = true;
+#if 0
+    if (bpp != 32)
+	*supported = false;
+#endif
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::GetVisibleRegion(BYTE *aRectangles, ULONG aCount, ULONG *aCountCopied)
+{
+    if (!aRectangles)
+        return E_POINTER;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::SetVisibleRegion(BYTE *aRectangles, ULONG aCount)
+{
+    if (!aRectangles)
+        return E_POINTER;
+    return S_OK;
+}
+
+STDMETHODIMP VBoxVNCFB::ProcessVHWACommand(BYTE *pCommand)
+{
+    return  E_NOTIMPL;
+}
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.h work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.h
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.h	1970-01-01 09:00:00.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/VNCFramebuffer.h	2010-01-23 18:55:20.000000000 +0900
@@ -0,0 +1,87 @@
+/* VNCFramebuffer.h */
+
+#ifndef __H_VNCFRAMEBUFFER
+#define __H_VNCFRAMEBUFFER
+
+#include <rfb/rfb.h>
+#include <rfb/keysym.h>
+
+#include <iprt/critsect.h>
+
+class VBoxVNCFB : VBOX_SCRIPTABLE_IMPL(IFramebuffer)
+{
+public:
+    VBoxVNCFB();
+    virtual ~VBoxVNCFB();
+
+#ifdef RT_OS_WINDOWS
+    STDMETHOD_(ULONG, AddRef)()
+    {
+        return ::InterlockedIncrement (&refcnt);
+    }
+    STDMETHOD_(ULONG, Release)()
+    {
+        long cnt = ::InterlockedDecrement (&refcnt);
+        if (cnt == 0)
+            delete this;
+        return cnt;
+    }
+#endif
+    VBOX_SCRIPTABLE_DISPATCH_IMPL(IFramebuffer)
+
+    NS_DECL_ISUPPORTS
+
+    STDMETHOD(COMGETTER(Width))(ULONG *width);
+    STDMETHOD(COMGETTER(Height))(ULONG *height);
+    STDMETHOD(Lock)();
+    STDMETHOD(Unlock)();
+    STDMETHOD(COMGETTER(Address))(BYTE **address);
+    STDMETHOD(COMGETTER(BitsPerPixel))(ULONG *bitsPerPixel);
+    STDMETHOD(COMGETTER(BytesPerLine))(ULONG *bytesPerLine);
+    STDMETHOD(COMGETTER(PixelFormat)) (ULONG *pixelFormat);
+    STDMETHOD(COMGETTER(UsesGuestVRAM)) (BOOL *usesGuestVRAM);
+    STDMETHOD(COMGETTER(HeightReduction)) (ULONG *heightReduction);
+    STDMETHOD(COMGETTER(Overlay)) (IFramebufferOverlay **aOverlay);
+    STDMETHOD(COMGETTER(WinId)) (uint64_t *winId);
+
+    STDMETHOD(NotifyUpdate)(ULONG x, ULONG y, ULONG w, ULONG h);
+    STDMETHOD(RequestResize)(ULONG aScreenId, ULONG pixelFormat, BYTE *vram,
+                             ULONG bitsPerPixel, ULONG bytesPerLine,
+                             ULONG w, ULONG h, BOOL *finished);
+    STDMETHOD(VideoModeSupported)(ULONG width, ULONG height, ULONG bpp, BOOL *supported);
+
+    STDMETHOD(GetVisibleRegion)(BYTE *aRectangles, ULONG aCount, ULONG *aCountCopied);
+    STDMETHOD(SetVisibleRegion)(BYTE *aRectangles, ULONG aCount);
+
+    STDMETHOD(ProcessVHWACommand)(BYTE *pCommand);
+
+    bool init(const char *name, unsigned int port, const char *address);
+    bool loadKeymap(const char *keymap);
+    void setAuthInfo(const char *secret);
+    const char *getAuthInfo() { return m_secret; }
+    void setMouseAbsolute(BOOL *AbsoluteMouseGuest, BOOL *GuestNeedsHostCursor);
+    bool getMouseAbsolute() {
+	return (*m_AbsoluteMouseGuest || *m_GuestNeedsHostCursor);
+    }
+    void setMousePtr(int x, int y) { m_ptrx = x; m_ptry = y; }
+    void getMousePtr(int &x, int &y) { x = m_ptrx; y = m_ptry; }
+    void setConsole(ComPtr<IConsole> &Console);
+    ComPtr<IKeyboard> getKeyboard() { return m_Keyboard; }
+    ComPtr<IMouse> getMouse() { return m_Mouse; }
+
+private:
+    rfbScreenInfoPtr m_Screen;
+    RTCRITSECT m_UpdateLock;
+    const char *m_secret;
+    BOOL *m_AbsoluteMouseGuest;
+    BOOL *m_GuestNeedsHostCursor;
+    ComPtr<IKeyboard> m_Keyboard;
+    ComPtr<IMouse> m_Mouse;
+    int m_ptrx, m_ptry;
+
+#ifdef RT_OS_WINDOWS
+    long refcnt;
+#endif
+};
+
+#endif // __H_VNCFRAMEBUFFER
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.c work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.c
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.c	1970-01-01 09:00:00.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.c	2010-01-23 22:32:38.000000000 +0900
@@ -0,0 +1,699 @@
+/* convert keysym to scancode */
+/* Created by Daisuke Aoyama <aoyama@peach.ne.jp> */
+
+#include <ctype.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <rfb/keysym.h>
+#include "keymap.h"
+
+#ifdef VBOX_WITH_LIBVNC
+#define KEYMAPDIR "/usr/local/share/virtualbox-ose/keymaps"
+#else
+#define KEYMAPDIR "/usr/share/syscons/keymaps"
+#endif /* VBOX_WITH_LIBVNC */
+#if 0
+#ifndef DEFAULT_KEYMAP
+#define DEFAULT_KEYMAP "builtin_us"
+#endif /* DEFAULT_KEYMAP */
+#endif
+
+#ifdef TEST
+#define KEYMAPFILE KEYMAPDIR "/us.pc-ctrl.kbd"
+/*
+#define KEYMAPFILE KEYMAPDIR "/jp.106.kbd"
+*/
+#endif /* TEST */
+
+typedef struct
+{
+    char *kbdstr;
+    int keysym;
+} kbdstr2keysym_t;
+
+typedef struct
+{
+    int scancode;
+    int keysym;
+} scancode2keysym_t;
+
+#define MAX_TMPBUF 4096
+#define XXX_NOTIMPL(label) 0
+
+static kbdstr2keysym_t kbdstr2keysym[] =
+{
+    { "nop",    0 },
+    { "bs",     XK_BackSpace },
+    { "ht",     XK_Tab },
+    { "",       XK_Linefeed },
+    { "",       XK_Clear },
+    { "cr",     XK_Return },
+    { "",       XK_Pause },
+    { "slock",  XK_Scroll_Lock },
+    { "",       XK_Sys_Req },
+    { "esc",    XK_Escape },
+    { "del",    XK_Delete },
+
+    { "fkey49", XK_Home },
+    { "fkey53", XK_Left },
+    { "fkey50", XK_Up },
+    { "fkey55", XK_Right },
+    { "fkey58", XK_Down },
+    { "",       XK_Prior },
+    { "fkey51", XK_Page_Up },
+    { "",       XK_Next },
+    { "fkey59", XK_Page_Down },
+    { "fkey57", XK_End },
+    { "",       XK_Begin },
+
+    { "",       XK_Select },
+    { "",       XK_Print },
+    { "",       XK_Execute },
+    { "fkey60", XK_Insert },
+    { "",       XK_Undo },
+    { "",       XK_Redo },
+    { "",       XK_Menu },
+    { "",       XK_Find },
+    { "",       XK_Cancel },
+    { "",       XK_Help },
+    { "",       XK_Break },
+    { "",       XK_Mode_switch },
+    { "",       XK_script_switch },
+    { "nlock",  XK_Num_Lock },
+
+    { "lshift", XK_Shift_L },
+    { "rshift", XK_Shift_R },
+    { "lctrl",  XK_Control_L },
+    { "rctrl",  XK_Control_R },
+    { "",       XK_Caps_Lock },
+    { "",       XK_Shift_Lock },
+
+    { "",       XK_Meta_L },
+    { "",       XK_Meta_R },
+    { "lalt",   XK_Alt_L },
+    { "ralt",   XK_Alt_R },
+    { "",       XK_Super_L },
+    { "",       XK_Super_R },
+    { "",       XK_Hyper_L },
+    { "",       XK_Hyper_R },
+
+    { "fkey01", XK_F1 },
+    { "fkey02", XK_F2 },
+    { "fkey03", XK_F3 },
+    { "fkey04", XK_F4 },
+    { "fkey05", XK_F5 },
+    { "fkey06", XK_F6 },
+    { "fkey07", XK_F7 },
+    { "fkey08", XK_F8 },
+    { "fkey09", XK_F9 },
+    { "fkey10", XK_F10 },
+    { "fkey11", XK_F11 },
+    { "",       XK_L1 },
+    { "fkey12", XK_F12 },
+    { "",       XK_L2 },
+    { "fkey13", XK_F13 },
+    { "",       XK_L3 },
+    { "fkey14", XK_F14 },
+    { "",       XK_L4 },
+    { "fkey15", XK_F15 },
+    { "",       XK_L5 },
+    { "fkey16", XK_F16 },
+    { "",       XK_L6 },
+    { "fkey17", XK_F17 },
+    { "",       XK_L7 },
+    { "fkey18", XK_F18 },
+    { "",       XK_L8 },
+    { "fkey19", XK_F19 },
+    { "",       XK_L9 },
+    { "fkey20", XK_F20 },
+    { "",       XK_L10 },
+    { "fkey21", XK_F21 },
+    { "",       XK_R1 },
+    { "fkey22", XK_F22 },
+    { "",       XK_R2 },
+    { "fkey23", XK_F23 },
+    { "",       XK_R3 },
+    { "fkey24", XK_F24 },
+    { "",       XK_R4 },
+    { "",       XK_F25 },
+    { "",       XK_R5 },
+    { "",       XK_F26 },
+    { "",       XK_R6 },
+    { "",       XK_F27 },
+    { "",       XK_R7 },
+    { "",       XK_F28 },
+    { "",       XK_R8 },
+    { "",       XK_F29 },
+    { "",       XK_R9 },
+    { "",       XK_F30 },
+    { "",       XK_R10 },
+    { "",       XK_F31 },
+    { "",       XK_R11 },
+    { "",       XK_F32 },
+    { "",       XK_R12 },
+    { "",       XK_F33 },
+    { "",       XK_R13 },
+    { "",       XK_F34 },
+    { "",       XK_R14 },
+    { "",       XK_F35 },
+    { "",       XK_R15 },
+
+#if 0
+    { "NUM_0",   XK_KP_0 },
+    { "NUM_1",   XK_KP_1 },
+    { "NUM_2",   XK_KP_2 },
+    { "NUM_3",   XK_KP_3 },
+    { "NUM_4",   XK_KP_4 },
+    { "NUM_5",   XK_KP_5 },
+    { "NUM_6",   XK_KP_6 },
+    { "NUM_7",   XK_KP_7 },
+    { "NUM_8",   XK_KP_8 },
+    { "NUM_9",   XK_KP_9 },
+
+    { "NUM_+",   XK_KP_Add },
+    { "NUM_-",   XK_KP_Subtract },
+    { "NUM_*",   XK_KP_Multiply },
+    { "NUM_/",   XK_KP_Divide },
+    { "NUM_.",   XK_KP_Decimal },
+    { "NUM_del", XK_KP_Delete },
+    { "NUM_cr",  XK_KP_Enter },
+
+    { "NUM_fkey49", XK_KP_Home },
+    { "NUM_fkey53", XK_KP_Left },
+    { "NUM_fkey50", XK_KP_Up },
+    { "NUM_fkey55", XK_KP_Right },
+    { "NUM_fkey58", XK_KP_Down },
+    { "",           XK_KP_Prior },
+    { "NUM_fkey51", XK_KP_Page_Up },
+    { "",           XK_KP_Next },
+    { "NUM_fkey59", XK_KP_Page_Down },
+    { "NUM_fkey57", XK_KP_End },
+    { "",           XK_KP_Begin },
+    { "NUM_fkey60", XK_KP_Insert },
+#endif
+
+    { NULL, -1 },
+};
+
+/* generate from /usr/share/syscons/keymaps/us.pc-ctrl.kbd */
+static scancode2keysym_t us_scancode2keysym [] =
+{
+    {   0, 0 },
+    {   0, 0 },
+    {   1, XK_Escape },
+    {   1, XK_Escape },
+    {   2, '1' },
+    {   2, '!' },
+    {   3, '2' },
+    {   3, '@' },
+    {   4, '3' },
+    {   4, '#' },
+    {   5, '4' },
+    {   5, '$' },
+    {   6, '5' },
+    {   6, '%' },
+    {   7, '6' },
+    {   7, '^' },
+    {   8, '7' },
+    {   8, '&' },
+    {   9, '8' },
+    {   9, '*' },
+    {  10, '9' },
+    {  10, '(' },
+    {  11, '0' },
+    {  11, ')' },
+    {  12, '-' },
+    {  12, '_' },
+    {  13, '=' },
+    {  13, '+' },
+    {  14, XK_BackSpace },
+    {  14, XK_BackSpace },
+    {  15, XK_Tab },
+    {  15, XXX_NOTIMPL(btab) },
+    {  16, 'q' },
+    {  16, 'Q' },
+    {  17, 'w' },
+    {  17, 'W' },
+    {  18, 'e' },
+    {  18, 'E' },
+    {  19, 'r' },
+    {  19, 'R' },
+    {  20, 't' },
+    {  20, 'T' },
+    {  21, 'y' },
+    {  21, 'Y' },
+    {  22, 'u' },
+    {  22, 'U' },
+    {  23, 'i' },
+    {  23, 'I' },
+    {  24, 'o' },
+    {  24, 'O' },
+    {  25, 'p' },
+    {  25, 'P' },
+    {  26, '[' },
+    {  26, '{' },
+    {  27, ']' },
+    {  27, '}' },
+    {  28, XK_Return },
+    {  28, XK_Return },
+    {  29, XK_Control_L },
+    {  29, XK_Control_L },
+    {  30, 'a' },
+    {  30, 'A' },
+    {  31, 's' },
+    {  31, 'S' },
+    {  32, 'd' },
+    {  32, 'D' },
+    {  33, 'f' },
+    {  33, 'F' },
+    {  34, 'g' },
+    {  34, 'G' },
+    {  35, 'h' },
+    {  35, 'H' },
+    {  36, 'j' },
+    {  36, 'J' },
+    {  37, 'k' },
+    {  37, 'K' },
+    {  38, 'l' },
+    {  38, 'L' },
+    {  39, ';' },
+    {  39, ':' },
+    {  40, '\'' },
+    {  40, '"' },
+    {  41, '`' },
+    {  41, '~' },
+    {  42, XK_Shift_L },
+    {  42, XK_Shift_L },
+    {  43, '\\' },
+    {  43, '|' },
+    {  44, 'z' },
+    {  44, 'Z' },
+    {  45, 'x' },
+    {  45, 'X' },
+    {  46, 'c' },
+    {  46, 'C' },
+    {  47, 'v' },
+    {  47, 'V' },
+    {  48, 'b' },
+    {  48, 'B' },
+    {  49, 'n' },
+    {  49, 'N' },
+    {  50, 'm' },
+    {  50, 'M' },
+    {  51, ',' },
+    {  51, '<' },
+    {  52, '.' },
+    {  52, '>' },
+    {  53, '/' },
+    {  53, '?' },
+    {  54, XK_Shift_R },
+    {  54, XK_Shift_R },
+    {  55, XK_KP_Multiply },
+    {  55, XK_KP_Multiply },
+    {  56, XK_Alt_L },
+    {  56, XK_Alt_L },
+    {  57, ' ' },
+    {  57, ' ' },
+    {  58, XK_Control_L },
+    {  58, XK_Control_L },
+    {  59, XK_F1 },
+    {  59, XK_F13 },
+    {  60, XK_F2 },
+    {  60, XK_F14 },
+    {  61, XK_F3 },
+    {  61, XK_F15 },
+    {  62, XK_F4 },
+    {  62, XK_F16 },
+    {  63, XK_F5 },
+    {  63, XK_F17 },
+    {  64, XK_F6 },
+    {  64, XK_F18 },
+    {  65, XK_F7 },
+    {  65, XK_F19 },
+    {  66, XK_F8 },
+    {  66, XK_F20 },
+    {  67, XK_F9 },
+    {  67, XK_F21 },
+    {  68, XK_F10 },
+    {  68, XK_F22 },
+    {  69, XK_Num_Lock },
+    {  69, XK_Num_Lock },
+    {  70, XK_Scroll_Lock },
+    {  70, XK_Scroll_Lock },
+    {  71, XK_Home },
+    {  71, XK_KP_7 },
+    {  72, XK_Up },
+    {  72, XK_KP_8 },
+    {  73, XK_Page_Up },
+    {  73, XK_KP_9 },
+    {  74, XXX_NOTIMPL(fkey52) },
+    {  74, XK_KP_Subtract },
+    {  75, XK_Left },
+    {  75, XK_KP_4 },
+    {  76, XXX_NOTIMPL(fkey54) },
+    {  76, XK_KP_5 },
+    {  77, XK_Right },
+    {  77, XK_KP_6 },
+    {  78, XXX_NOTIMPL(fkey56) },
+    {  78, XK_KP_Add },
+    {  79, XK_End },
+    {  79, XK_KP_1 },
+    {  80, XK_Down },
+    {  80, XK_KP_2 },
+    {  81, XK_Page_Down },
+    {  81, XK_KP_3 },
+    {  82, XK_Insert },
+    {  82, XK_KP_0 },
+    {  83, XK_Delete },
+    {  83, XK_KP_Decimal },
+    {  84, 0 },
+    {  84, 0 },
+    {  85, 0 },
+    {  85, 0 },
+    {  86, 0 },
+    {  86, 0 },
+    {  87, XK_F11 },
+    {  87, XK_F23 },
+    {  88, XK_F12 },
+    {  88, XK_F24 },
+    {  89, XK_Return },
+    {  89, XK_Return },
+    {  90, XK_Control_R },
+    {  90, XK_Control_R },
+    {  91, XK_KP_Divide },
+    {  91, XK_KP_Divide },
+    {  92, XXX_NOTIMPL(nscr) },
+    {  92, XXX_NOTIMPL(pscr) },
+    {  93, XK_Alt_R },
+    {  93, XK_Alt_R },
+    {  94, XK_Home },
+    {  94, XK_Home },
+    {  95, XK_Up },
+    {  95, XK_Up },
+    {  96, XK_Page_Up },
+    {  96, XK_Page_Up },
+    {  97, XK_Left },
+    {  97, XK_Left },
+    {  98, XK_Right },
+    {  98, XK_Right },
+    {  99, XK_End },
+    {  99, XK_End },
+    { 100, XK_Down },
+    { 100, XK_Down },
+    { 101, XK_Page_Down },
+    { 101, XK_Page_Down },
+    { 102, XK_Insert },
+    { 102, XXX_NOTIMPL(paste) },
+    { 103, XXX_NOTIMPL(fkey61) },
+    { 103, XXX_NOTIMPL(fkey61) },
+    { 104, XK_Scroll_Lock },
+    { 104, XXX_NOTIMPL(saver) },
+    { 105, XXX_NOTIMPL(fkey62) },
+    { 105, XXX_NOTIMPL(fkey62) },
+    { 106, XXX_NOTIMPL(fkey63) },
+    { 106, XXX_NOTIMPL(fkey63) },
+    { 107, XXX_NOTIMPL(fkey64) },
+    { 107, XXX_NOTIMPL(fkey64) },
+    { 108, 0 },
+    { 108, 0 },
+
+    { -1, -1 },
+};
+
+/* current keymap */
+static scancode2keysym_t *scancode2keysym = &us_scancode2keysym[0];
+
+static int
+parse_keymap_line(char *buf, int *scancode, char *chB, size_t sizeB,
+		  char *chS, size_t sizeS)
+{
+    const char *p = buf;
+    const char *q;
+    int lockstate = 0;
+
+    /*
+      input format:
+        "# code  base   shift  cntrl  shift ... lockstate"
+        "  001   esc    esc    esc    esc ... O"
+        "  002   '1'    '!'    nop    nop ... O"
+    */
+
+    /* loclstate */
+    q = strchr(p, '\0');
+    while(q > p && isspace(*--q))
+	;
+#if 0
+    lockstate = *q;
+    if (lockstate == 0)
+	lockstate = 'O';
+#else
+    lockstate = 'O';
+#endif
+
+    /* scan code */
+    while (isspace(*p))
+	p++;
+    if (sscanf(p, "%d", scancode) != 1) {
+	return -1;
+    }
+    while (!isspace(*p))
+	p++;
+
+    /* base */
+    while (isspace(*p))
+	p++;
+    if (*p == '\'' && *(p+1) != '\0' && *(p+2) == '\'') {
+	/* letter */
+	q = p + 3;
+    } else {
+	/* string */
+	q = p;
+	while (!isspace(*q))
+	    q++;
+    }
+    snprintf(chB, sizeB, "%c%.*s", lockstate, q - p, p);
+    p = q;
+
+    /* shift */
+    while (isspace(*p))
+	p++;
+    if (*p == '\'' && *(p+1) != '\0' && *(p+2) == '\'') {
+	/* letter */
+	q = p + 3;
+    } else {
+	/* string */
+	q = p;
+	while (!isspace(*q))
+	    q++;
+    }
+    snprintf(chS, sizeS, "%c%.*s", lockstate, q - p, p);
+    p = q;
+
+    return 0;
+}
+
+static int
+str2keysym(const char *str)
+{
+    const char *p = str + 1;
+    const char *q = str;
+    char buf[MAX_TMPBUF];
+    int keysym;
+    int i;
+
+    if (*p == '\'' && *(p+1) != '\0' && *(p+2) == '\'') {
+	/* letter */
+	keysym = (int)*(p+1);
+	if (*q == 'N') {
+	    snprintf(buf, sizeof buf, "NUM_%c", keysym);
+	    goto search_keysym;
+	}
+    } else {
+	/* string */
+	keysym = 0;
+	if (*q == 'N') {
+	    snprintf(buf, sizeof buf, "NUM_%s", p);
+	    p = buf;
+	}
+    search_keysym:
+	for (i = 0; kbdstr2keysym[i].keysym != -1; i++) {
+	    if (kbdstr2keysym[i].kbdstr == NULL
+		|| kbdstr2keysym[i].kbdstr[0] == '\0')
+		continue;
+	    if (strcasecmp(kbdstr2keysym[i].kbdstr, p) == 0) {
+		keysym = kbdstr2keysym[i].keysym;
+		break;
+	    }
+	}
+    }
+
+    return keysym;
+}
+
+static scancode2keysym_t *
+create_scancode_table(const char *file)
+{
+    scancode2keysym_t *sc2ks = NULL;
+    FILE *fp = NULL;
+    char buf[MAX_TMPBUF];
+    char bufchB[MAX_TMPBUF];
+    char bufchS[MAX_TMPBUF];
+    int lineno = 0;
+    int num = 0;
+    int scancode;
+
+    fp = fopen(file, "r");
+    if (fp == NULL) {
+	fprintf(stderr, "open error: %s\n", file);
+	goto error;
+    }
+    while (fgets(buf, sizeof buf, fp) != NULL) {
+	lineno++;
+	if (buf[0] == '#' || buf[0] =='\n' || buf[0] == '\0')
+	    continue;
+	if (parse_keymap_line(buf, &scancode, bufchB, sizeof bufchB,
+			      bufchS, sizeof bufchS) != 0) {
+	    fprintf(stderr, "parse error at line %d.\n", lineno);
+	    goto error;
+	}
+#if 0
+	printf("scancode=%d, chB=%s, chS=%s\n", scancode, bufchB, bufchS);
+#endif
+	/* allocate for base and shift scancode */
+	num++;
+	if (num == 1) {
+	    sc2ks = malloc(sizeof *sc2ks * (num * 2));
+	    if (sc2ks == NULL) {
+		fprintf(stderr, "malloc error.\n");
+		goto error;
+	    }
+	} else {
+	    sc2ks = realloc(sc2ks, sizeof *sc2ks * (num * 2));
+	    if (sc2ks == NULL) {
+		fprintf(stderr, "realloc error.\n");
+		goto error;
+	    }
+	}
+
+	/* chB */
+	sc2ks[(num - 1) * 2 + 0].scancode = scancode;
+	sc2ks[(num - 1) * 2 + 0].keysym = str2keysym(bufchB);
+	/* chS */
+	sc2ks[(num - 1) * 2 + 1].scancode = scancode;
+	sc2ks[(num - 1) * 2 + 1].keysym = str2keysym(bufchS);
+    }
+    /* table terminater */
+    if (num == 0) {
+	fprintf(stderr, "no keymap.\n");
+	goto error;
+    } else {
+	sc2ks = realloc(sc2ks, sizeof *sc2ks * ((num * 2) + 1));
+	sc2ks[num * 2].scancode = -1;
+	sc2ks[num * 2].keysym = -1;
+    }
+    fclose(fp);
+    return sc2ks;
+
+error:
+    if (fp != NULL)
+	fclose(fp);
+    if (sc2ks != NULL)
+	free(sc2ks);
+    return NULL;
+}
+
+int
+load_keymap_table(const char *file)
+{
+    scancode2keysym_t *sc2ks = NULL;
+    char fullpath[FILENAME_MAX];
+
+    if (file == NULL
+	|| strcasecmp(file, "default") == 0) {
+#ifdef DEFAULT_KEYMAP
+	file = DEFAULT_KEYMAP;
+#else
+	sc2ks = &us_scancode2keysym[0];
+	scancode2keysym = sc2ks;
+	return 0;
+#endif /* DEFAULT_KEYMAP */
+    }
+
+    if (strcasecmp(file, "builtin_us") == 0) {
+	sc2ks = &us_scancode2keysym[0];
+    } else {
+	if (file[0] != '/') {
+	    snprintf(fullpath, sizeof fullpath, "%s/%s",
+		     KEYMAPDIR, file);
+	    sc2ks = create_scancode_table(fullpath);
+	} else {
+	    sc2ks = create_scancode_table(file);
+	}
+    }
+    if (sc2ks == NULL) {
+	fprintf(stderr, "create table failed.\n");
+	return -1;
+    }
+    scancode2keysym = sc2ks;
+    return 0;
+}
+
+int
+keysym2scancode(int keysym)
+{
+    int i;
+    
+    for (i = 0; scancode2keysym[i].keysym != -1; i++) {
+	if (scancode2keysym[i].keysym == 0) {
+	    /* nop or notimpl */
+	    continue;
+	}
+	if (scancode2keysym[i].keysym == keysym) {
+	    return scancode2keysym[i].scancode;
+	}
+    }
+    return -1;
+}
+
+#ifdef TEST
+static void
+dump_keymap_table(const scancode2keysym_t *sc2ks)
+{
+    int i;
+
+    if (sc2ks == NULL)
+	return;
+    printf("keymap table:\n");
+    for (i = 0; sc2ks[i].keysym != -1; i++) {
+	printf("%d: scancode=%d, keysym=%x\n",
+	       i, sc2ks[i].scancode, sc2ks[i].keysym);
+    }
+}
+
+int
+main(int argc, char **argv)
+{
+    char *file = KEYMAPFILE;
+    int scancode;
+
+    if (argc > 1) {
+	file = argv[1];
+    }
+    printf("load: %s\n", file);
+
+    if (load_keymap_table(file) != 0) {
+	fprintf(stderr, "load keymap failed.\n");
+	return 1;
+    }
+#if 0
+    dump_keymap_table(scancode2keysym);
+#endif
+    scancode = keysym2scancode(XK_Delete);
+    printf("XK_Delete = %x\n", scancode);
+    scancode = keysym2scancode('@');
+    printf("'@' = %x\n", scancode);
+    return 0;
+}
+#endif /* TEST */
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.h work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.h
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.h	1970-01-01 09:00:00.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Frontends/VBoxHeadless/keymap.h	2010-01-23 18:55:28.746546000 +0900
@@ -0,0 +1,14 @@
+/* keymap.h */
+
+#ifndef KEYMAP_H
+#define KEYMAP_H
+
+#if defined(__cplusplus)
+extern "C" {
+#endif
+int load_keymap_table(const char *file);
+int keysym2scancode(int keysym);
+#if defined(__cplusplus)
+}
+#endif
+#endif/* KEYMAP_H */
diff -aurN work-extract/VirtualBox-3.1.2_OSE/src/VBox/Devices/testcase/tstDeviceStructSize.cpp work/VirtualBox-3.1.2_OSE/src/VBox/Devices/testcase/tstDeviceStructSize.cpp
--- work-extract/VirtualBox-3.1.2_OSE/src/VBox/Devices/testcase/tstDeviceStructSize.cpp	2009-12-17 23:27:55.000000000 +0900
+++ work/VirtualBox-3.1.2_OSE/src/VBox/Devices/testcase/tstDeviceStructSize.cpp	2010-01-16 19:45:50.000000000 +0900
@@ -301,9 +301,11 @@
 #endif
     CHECK_MEMBER_ALIGNMENT(PITState, StatPITIrq, 8);
     CHECK_MEMBER_ALIGNMENT(SerialState, CritSect, 8);
+#if 0
     CHECK_MEMBER_ALIGNMENT(VGASTATE, Dev, 8);
     CHECK_MEMBER_ALIGNMENT(VGASTATE, lock, 8);
     CHECK_MEMBER_ALIGNMENT(VGASTATE, StatRZMemoryRead, 8);
+#endif
     CHECK_MEMBER_ALIGNMENT(VMMDevState, CritSect, 8);
 #ifdef VBOX_WITH_VIRTIO
     CHECK_MEMBER_ALIGNMENT(VPCISTATE, cs, 8);
